------------------------------------------------------------------------------
-- user_logic.vhd - entity/architecture pair
------------------------------------------------------------------------------
--
-- ***************************************************************************
-- ** Copyright (c) 1995-2010 Xilinx, Inc.  All rights reserved.            **
-- **                                                                       **
-- ** Xilinx, Inc.                                                          **
-- ** XILINX IS PROVIDING THIS DESIGN, CODE, OR INFORMATION "AS IS"         **
-- ** AS A COURTESY TO YOU, SOLELY FOR USE IN DEVELOPING PROGRAMS AND       **
-- ** SOLUTIONS FOR XILINX DEVICES.  BY PROVIDING THIS DESIGN, CODE,        **
-- ** OR INFORMATION AS ONE POSSIBLE IMPLEMENTATION OF THIS FEATURE,        **
-- ** APPLICATION OR STANDARD, XILINX IS MAKING NO REPRESENTATION           **
-- ** THAT THIS IMPLEMENTATION IS FREE FROM ANY CLAIMS OF INFRINGEMENT,     **
-- ** AND YOU ARE RESPONSIBLE FOR OBTAINING ANY RIGHTS YOU MAY REQUIRE      **
-- ** FOR YOUR IMPLEMENTATION.  XILINX EXPRESSLY DISCLAIMS ANY              **
-- ** WARRANTY WHATSOEVER WITH RESPECT TO THE ADEQUACY OF THE               **
-- ** IMPLEMENTATION, INCLUDING BUT NOT LIMITED TO ANY WARRANTIES OR        **
-- ** REPRESENTATIONS THAT THIS IMPLEMENTATION IS FREE FROM CLAIMS OF       **
-- ** INFRINGEMENT, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS       **
-- ** FOR A PARTICULAR PURPOSE.                                             **
-- **                                                                       **
-- ***************************************************************************
--
------------------------------------------------------------------------------
-- Filename:          user_logic.vhd
-- Version:           1.01.a
-- Description:       User logic.
-- Date:              Sat Feb 12 15:07:14 2011 (by Create and Import Peripheral Wizard)
-- VHDL Standard:     VHDL'93
------------------------------------------------------------------------------
-- Naming Conventions:
--   active low signals:                    "*_n"
--   clock signals:                         "clk", "clk_div#", "clk_#x"
--   reset signals:                         "rst", "rst_n"
--   generics:                              "C_*"
--   user defined types:                    "*_TYPE"
--   state machine next state:              "*_ns"
--   state machine current state:           "*_cs"
--   combinatorial signals:                 "*_com"
--   pipelined or register delay signals:   "*_d#"
--   counter signals:                       "*cnt*"
--   clock enable signals:                  "*_ce"
--   internal version of output port:       "*_i"
--   device pins:                           "*_pin"
--   ports:                                 "- Names begin with Uppercase"
--   processes:                             "*_PROCESS"
--   component instantiations:              "<ENTITY_>I_<#|FUNC>"
------------------------------------------------------------------------------

-- DO NOT EDIT BELOW THIS LINE --------------------
library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_arith.all;
use ieee.std_logic_unsigned.all;

library proc_common_v3_00_a;
use proc_common_v3_00_a.proc_common_pkg.all;

-- DO NOT EDIT ABOVE THIS LINE --------------------

--USER libraries added here
Library UNISIM;
use UNISIM.vcomponents.all;

Library UNIMACRO;
use UNIMACRO.vcomponents.all;


------------------------------------------------------------------------------
-- Entity section
------------------------------------------------------------------------------
-- Definition of Generics:
--   C_SLV_DWIDTH                 -- Slave interface data bus width
--   C_NUM_REG                    -- Number of software accessible registers
--
-- Definition of Ports:
--   Bus2IP_Clk                   -- Bus to IP clock
--   Bus2IP_Reset                 -- Bus to IP reset
--   Bus2IP_Data                  -- Bus to IP data bus
--   Bus2IP_BE                    -- Bus to IP byte enables
--   Bus2IP_RdCE                  -- Bus to IP read chip enable
--   Bus2IP_WrCE                  -- Bus to IP write chip enable
--   IP2Bus_Data                  -- IP to Bus data bus
--   IP2Bus_RdAck                 -- IP to Bus read transfer acknowledgement
--   IP2Bus_WrAck                 -- IP to Bus write transfer acknowledgement
--   IP2Bus_Error                 -- IP to Bus error response
------------------------------------------------------------------------------

entity user_logic is
  generic
  (
    -- ADD USER GENERICS BELOW THIS LINE ---------------
    -- ADD USER GENERICS ABOVE THIS LINE ---------------

    -- DO NOT EDIT BELOW THIS LINE ---------------------
    -- Bus protocol parameters, do not add to or delete
    C_SLV_DWIDTH                   : integer              := 32;
    C_NUM_REG                      : integer              := 6
    -- DO NOT EDIT ABOVE THIS LINE ---------------------
  );
  port
  (
    -- ADD USER PORTS BELOW THIS LINE ------------------
    rd_fifo_clk               : in std_logic;
    rd_fifo_rd_en             : in std_logic;
    rd_fifo_data              : out std_logic_vector(31 downto 0); 
    rd_fifo_empty             : out std_logic;
    rd_fifo_full              : out std_logic; 
    npi_rd_ready              : out std_logic; 


    wr_fifo_clk               : in std_logic;
    wr_fifo_wr_en             : in std_logic;
    wr_fifo_data              : in std_logic_vector(31 downto 0); 
    wr_fifo_empty             : out std_logic;
    wr_fifo_full              : out std_logic; 
    npi_wr_ready              : out std_logic; 

    XIL_NPI_Addr              : out std_logic_vector(31 downto 0)     	             := (others => '0');
    XIL_NPI_AddrReq           : out std_logic                                        := '0';
    XIL_NPI_AddrAck           : in  std_logic                                        := '0';
    XIL_NPI_RNW               : out std_logic                                        := '0';
    XIL_NPI_Size              : out std_logic_vector(3 downto 0)                     := (others => '0');
    XIL_NPI_WrFIFO_Data       : out std_logic_vector(64 -1 downto 0)     := (others => '0');
    XIL_NPI_WrFIFO_BE         : out std_logic_vector((64)/8-1 downto 0) := (others => '0');
    XIL_NPI_WrFIFO_Push       : out std_logic                                        := '0';
    XIL_NPI_RdFIFO_Data       : in  std_logic_vector(64-1 downto 0)     := (others => '0');
    XIL_NPI_RdFIFO_Pop        : out std_logic                                        := '0';
    XIL_NPI_RdFIFO_RdWdAddr   : in  std_logic_vector(3 downto 0)                     := (others => '0');
    XIL_NPI_WrFIFO_Empty      : in  std_logic                                        := '0';
    XIL_NPI_WrFIFO_AlmostFull : in  std_logic                                        := '0';
    XIL_NPI_WrFIFO_Flush      : out std_logic                                        := '0';
    XIL_NPI_RdFIFO_Empty      : in  std_logic                                        := '0';
    XIL_NPI_RdFIFO_Flush      : out std_logic                                        := '0';
    XIL_NPI_RdFIFO_Latency    : in  std_logic_vector(1 downto 0)                     := (others => '0');
    XIL_NPI_RdModWr           : out std_logic                                        := '0';
    XIL_NPI_InitDone          : in  std_logic                                        := '0';
    XIL_NPI_Clk               : in  std_logic                                        := '0';
    XIL_NPI_Rst               : in  std_logic                                        := '0';

    -- ADD USER PORTS ABOVE THIS LINE ------------------

    -- DO NOT EDIT BELOW THIS LINE ---------------------
    -- Bus protocol ports, do not add to or delete
    Bus2IP_Clk                     : in  std_logic;
    Bus2IP_Reset                   : in  std_logic;
    Bus2IP_Data                    : in  std_logic_vector(0 to C_SLV_DWIDTH-1);
    Bus2IP_BE                      : in  std_logic_vector(0 to C_SLV_DWIDTH/8-1);
    Bus2IP_RdCE                    : in  std_logic_vector(0 to C_NUM_REG-1);
    Bus2IP_WrCE                    : in  std_logic_vector(0 to C_NUM_REG-1);
    IP2Bus_Data                    : out std_logic_vector(0 to C_SLV_DWIDTH-1);
    IP2Bus_RdAck                   : out std_logic;
    IP2Bus_WrAck                   : out std_logic;
    IP2Bus_Error                   : out std_logic
    -- DO NOT EDIT ABOVE THIS LINE ---------------------
  );

  attribute SIGIS : string;
  attribute SIGIS of Bus2IP_Clk    : signal is "CLK";
  attribute SIGIS of Bus2IP_Reset  : signal is "RST";

end entity user_logic;
------------------------------------------------------------------------------
-- Architecture section
------------------------------------------------------------------------------

architecture IMP of user_logic is
	----------------------------------------------------------
	signal fifo_rst : std_logic := '0';
	----------------------------------------------------------
	constant start_wr_cmd_patt	: std_logic_vector(7 downto 0) := X"66";	
	constant start_rd_cmd_patt	: std_logic_vector(7 downto 0) := X"55";
	constant user_rst_cmd_patt	: std_logic_vector(7 downto 0) := X"9A";
	signal start_wr_cmd, start_rd_cmd, user_rst_cmd : std_logic;
	----------------------------------------------------------
	signal wr_fifo_rd_en : std_logic;
	signal wr_fifo_dout : std_logic_vector((64- 1)  downto 0);
	signal wr_fifo_prog_empty_s : std_logic;
	signal wr_fifo_empty_s, wr_fifo_full_s, rd_fifo_empty_s, rd_fifo_full_s : std_logic := '0';
	----------------------------------------------------------
	signal burst_cnt : std_logic_vector(3 downto 0) := x"0";
	signal add_cnt, c_burst_repeat_cnt : std_logic_vector(15 downto 0) := (others => '0');
	signal c_again_cnt, g_cnt : std_logic_vector(31 downto 0) := (others => '0');
	----------------------------------------------------------
	signal rd_fifo_prog_full_s : std_logic;
	signal rd_fifo_wr_en : std_logic := '0';
	signal rd_fifo_din : std_logic_vector((64 - 1) downto 0); 
	signal XIL_NPI_RdFIFO_Pop_s, XIL_NPI_RdFIFO_Pop_d0, XIL_NPI_RdFIFO_Pop_d1 : std_logic;
	----------------------------------------------------------
	type pim_state_type is (
		st_fifo_rst3clk, 
		st_idle, 
		st_mpmc_rd_flush_rst, 
		st_mpmc_wr_flush_rst, 
		st_mpmc_rd_flush_rst_wait ,
		st_mpmc_wr_flush_rst_wait, 
		st_wait_command, 
		st_rd_pop_data, 
		st_rd_wait_rdy, 
		st_rd_add_req, 
		st_rd_next, 
		st_rd_finish, 
		st_wr_wait_rdy, 
		st_wr_wait_full, 
		st_wr_data_push, 
		st_wr_add_req_ack, 
		st_wr_wait_empty_high);

	signal pim_state : pim_state_type;

	------------------------------------------
	-- Signals for user logic slave model s/w accessible register example
	------------------------------------------
	signal slv_wr_reg0                    : std_logic_vector(0 to C_SLV_DWIDTH-1);

	signal slv_rd_reg0                    : std_logic_vector(0 to C_SLV_DWIDTH-1);
	signal slv_rd_reg1                    : std_logic_vector(0 to C_SLV_DWIDTH-1);
	signal slv_rd_reg2                    : std_logic_vector(0 to C_SLV_DWIDTH-1);
	signal slv_rd_reg3                    : std_logic_vector(0 to C_SLV_DWIDTH-1);
	signal slv_rd_reg4                    : std_logic_vector(0 to C_SLV_DWIDTH-1);
	signal slv_rd_reg5                    : std_logic_vector(0 to C_SLV_DWIDTH-1) := x"0000201a";	-- npi user logic indicator
	signal pim_status                     : std_logic_vector(7 downto 0);

	signal slv_reg_write_sel              : std_logic_vector(0 to 5);
	signal slv_reg_read_sel               : std_logic_vector(0 to 5);
	signal slv_ip2bus_data                : std_logic_vector(0 to C_SLV_DWIDTH-1);
	signal slv_read_ack                   : std_logic;
	signal slv_write_ack                  : std_logic;
	signal cnt_fifo_rst3clk				: std_logic_vector(3 downto 0); 

	signal built_in_rdcount_rd, built_in_wrcount_rd : std_logic_vector(9 downto 0);
	signal built_in_rdcount_wr, built_in_wrcount_wr : std_logic_vector(9 downto 0);
	signal dummy0, dummy1, dummy2, dummy3 : std_logic;
	signal npi_rd_ready_s, npi_wr_ready_s : std_logic;

	component afifo_bram_v5_wr32_rd64
		port (
		rst: IN std_logic;
		wr_clk: IN std_logic;
		rd_clk: IN std_logic;
		din: IN std_logic_VECTOR(31 downto 0);
		wr_en: IN std_logic;
		rd_en: IN std_logic;
		dout: OUT std_logic_VECTOR(63 downto 0);
		full: OUT std_logic;
		empty: OUT std_logic;
		prog_empty: OUT std_logic);
	end component;

	component afifo_bram_v5_rd32_wr64
		port (
		rst: IN std_logic;
		wr_clk: IN std_logic;
		rd_clk: IN std_logic;
		din: IN std_logic_VECTOR(63 downto 0);
		wr_en: IN std_logic;
		rd_en: IN std_logic;
		dout: OUT std_logic_VECTOR(31 downto 0);
		full: OUT std_logic;
		empty: OUT std_logic;
		prog_full: OUT std_logic);
	end component;

	component afifo_bram_v6_wr32_rd64
		port (
		rst: IN std_logic;
		wr_clk: IN std_logic;
		rd_clk: IN std_logic;
		din: IN std_logic_VECTOR(31 downto 0);
		wr_en: IN std_logic;
		rd_en: IN std_logic;
		dout: OUT std_logic_VECTOR(63 downto 0);
		full: OUT std_logic;
		empty: OUT std_logic;
		prog_empty: OUT std_logic);
	end component;

	component afifo_bram_v6_rd32_wr64
		port (
		rst: IN std_logic;
		wr_clk: IN std_logic;
		rd_clk: IN std_logic;
		din: IN std_logic_VECTOR(63 downto 0);
		wr_en: IN std_logic;
		rd_en: IN std_logic;
		dout: OUT std_logic_VECTOR(31 downto 0);
		full: OUT std_logic;
		empty: OUT std_logic;
		prog_full: OUT std_logic);
	end component;


begin
	slv_reg_write_sel <= Bus2IP_WrCE(0 to 5);
	slv_reg_read_sel  <= Bus2IP_RdCE(0 to 5);
	slv_write_ack     <= Bus2IP_WrCE(0) or Bus2IP_WrCE(1) or Bus2IP_WrCE(2) or Bus2IP_WrCE(3) or Bus2IP_WrCE(4) or Bus2IP_WrCE(5);
	slv_read_ack      <= Bus2IP_RdCE(0) or Bus2IP_RdCE(1) or Bus2IP_RdCE(2) or Bus2IP_RdCE(3) or Bus2IP_RdCE(4) or Bus2IP_RdCE(5);

	-- implement slave model software accessible register(s)
	SLAVE_REG_WRITE_PROC : process( Bus2IP_Clk ) is
	begin
		if Bus2IP_Clk'event and Bus2IP_Clk = '1' then
			if Bus2IP_Reset = '1' then
				slv_wr_reg0 <= (others => '0');
			else
				case slv_reg_write_sel is
					when "100000" => slv_wr_reg0 <= Bus2IP_Data;		
					when "010000" => c_burst_repeat_cnt <= Bus2IP_Data(16 to 31);	-- burst repeat cnt
					when "001000" => c_again_cnt <= Bus2IP_Data(0 to 31);	-- cnt for how many times go back and start agin 
					when others => null;
				end case;
			end if;
		end if;
	end process SLAVE_REG_WRITE_PROC;

	-- implement slave model software accessible register(s) read mux
	SLAVE_REG_READ_PROC : process( slv_reg_read_sel, slv_rd_reg0, slv_rd_reg1, slv_rd_reg2, slv_rd_reg3, slv_rd_reg4, slv_rd_reg5 ) is
	begin
		case slv_reg_read_sel is
			when "100000" => slv_ip2bus_data <= slv_rd_reg0;
			when "010000" => slv_ip2bus_data <= slv_rd_reg1;
			when "001000" => slv_ip2bus_data <= slv_rd_reg2;
			when "000100" => slv_ip2bus_data <= slv_rd_reg3;
			when "000010" => slv_ip2bus_data <= slv_rd_reg4;
			when "000001" => slv_ip2bus_data <= slv_rd_reg5;
			when others => slv_ip2bus_data <= (others => '0');
		end case;
	end process SLAVE_REG_READ_PROC;

	------------------------------------------
	-- Example code to drive IP to Bus signals
	------------------------------------------
	IP2Bus_Data  <= slv_ip2bus_data when slv_read_ack = '1' else (others => '0');

	IP2Bus_WrAck <= slv_write_ack;
	IP2Bus_RdAck <= slv_read_ack;
	IP2Bus_Error <= '0';

	wr_fifo_full <= wr_fifo_full_s;
	wr_fifo_empty <= wr_fifo_empty_s;

	wr_fifo_to_mpmc : afifo_bram_v6_wr32_rd64 
  	port map (
		rst => fifo_rst,
		wr_clk => wr_fifo_clk,
		wr_en => wr_fifo_wr_en,
		din => wr_fifo_data,
		rd_clk => XIL_NPI_Clk,
		rd_en => wr_fifo_rd_en,
		dout => wr_fifo_dout,
		empty => wr_fifo_empty_s,
		full => wr_fifo_full_s,
		prog_empty => wr_fifo_prog_empty_s);


	rd_fifo_empty <= rd_fifo_empty_s;                          
	rd_fifo_full  <= rd_fifo_full_s;

	rd_fifo_from_mpmc : afifo_bram_v6_rd32_wr64 
	port map (
		rst => fifo_rst,
		rd_clk => rd_fifo_clk,
		rd_en => rd_fifo_rd_en,
		dout => rd_fifo_data,
		wr_clk => XIL_NPI_Clk,
		wr_en => rd_fifo_wr_en,
		din => rd_fifo_din,
		empty => rd_fifo_empty_s,
		full => rd_fifo_full_s,
		prog_full => rd_fifo_prog_full_s);

	process(XIL_NPI_Clk)
	begin
		if XIL_NPI_Clk'event and XIL_NPI_Clk = '1' then
			XIL_NPI_WrFIFO_Push <= wr_fifo_rd_en;
		end if;
	end process;

	process(XIL_NPI_Clk)
	begin
		if XIL_NPI_Clk'event and XIL_NPI_Clk = '1' then
			if  user_rst_cmd = '1' or XIL_NPI_Rst = '1' then 
				pim_state <= st_fifo_rst3clk; 
				cnt_fifo_rst3clk <= (others => '0');
				XIL_NPI_AddrReq <= '0';
				XIL_NPI_RNW <= '1';
				XIL_NPI_RDMODWR <= '1';
				wr_fifo_rd_en <= '0';
				npi_rd_ready_s <= '0';              
				npi_wr_ready_s <= '0';              
				fifo_rst <= '1';
				XIL_NPI_WrFIFO_Flush <= '0';      
				XIL_NPI_RdFIFO_Flush <= '0';      
			else
				case pim_state is
				when st_fifo_rst3clk => 
					cnt_fifo_rst3clk <= cnt_fifo_rst3clk + 1;
					if(cnt_fifo_rst3clk = x"a") then
						pim_state <= st_idle;
					end if;
				when st_idle => 
					pim_status <= x"00";
					add_cnt <= (others => '0');
					g_cnt <= (others => '0');
					fifo_rst <= '0';
					npi_rd_ready_s <= '0';              
					npi_wr_ready_s <= '0';              
					if(XIL_NPI_InitDone = '1' and wr_fifo_full_s = '0' and rd_fifo_full_s = '0' ) then
						--fifo_rst <= '0';
						pim_state <= st_mpmc_rd_flush_rst;
					end if;
				when st_mpmc_rd_flush_rst =>
					pim_status <= x"01";
					if XIL_NPI_RdFIFO_Empty = '0' then
						XIL_NPI_RdFIFO_Flush <= '1';      
						pim_state <= st_mpmc_rd_flush_rst_wait;
					else
						pim_state <= st_mpmc_wr_flush_rst;
					end if;
				when st_mpmc_rd_flush_rst_wait =>
					pim_status <= x"02";
					XIL_NPI_RdFIFO_Flush <= '0';      
					if XIL_NPI_RdFIFO_Empty = '1' then
						pim_state <= st_mpmc_wr_flush_rst;
					end if;
				when st_mpmc_wr_flush_rst =>
					pim_status <= x"03";
					if XIL_NPI_WrFIFO_Empty = '0' then
						XIL_NPI_WrFIFO_Flush <= '1';      
						pim_state <= st_mpmc_wr_flush_rst_wait;
					else
						pim_state <= st_wait_command;
					end if;
				when st_mpmc_wr_flush_rst_wait =>
					pim_status <= x"04";
					XIL_NPI_WrFIFO_Flush <= '0';      
					if XIL_NPI_WrFIFO_Empty = '1' then
						pim_state <= st_wait_command;
					end if;
				-----------------------------------------------------------------------------
				-----------------------------------------------------------------------------
				-----------------------------------------------------------------------------
				when st_wait_command => 
					pim_status <= x"05";
					if start_rd_cmd = '1' then
						npi_rd_ready_s <= '1';
						pim_state <= st_rd_wait_rdy;
					elsif start_wr_cmd = '1' then
						npi_wr_ready_s <= '1';
						pim_state <= st_wr_wait_rdy;
					end if;
				-----------------------------------------------------------------------------
				-----------------------------------------------------------------------------
				-----------------------------------------------------------------------------
				when st_wr_wait_rdy => 
					pim_status <= x"06";
					if XIL_NPI_WrFIFO_EMPTY = '1' then		--- if pim is ready
						pim_state <= st_wr_wait_full;
					end if;
				when st_wr_wait_full =>
					pim_status <= x"07";
					if wr_fifo_prog_empty_s = '0' then	--- if wr_fifo has enough data to write
						burst_cnt <= x"f";
						wr_fifo_rd_en <= '1';
						pim_state <= st_wr_data_push;
					end if;
				when st_wr_data_push =>
					pim_status <= x"08";
					if XIL_NPI_WrFIFO_ALMOSTFULL = '0' then
						burst_cnt <= burst_cnt- 1;
						if(burst_cnt = 0) then
							wr_fifo_rd_en <= '0';
							XIL_NPI_AddrReq <= '1';
							XIL_NPI_RNW<= '0';
							XIL_NPI_RDMODWR <= '0';
							pim_state <= st_wr_add_req_ack;
						end if;
					end if;
				when st_wr_add_req_ack =>
					pim_status <= x"09";
					if XIL_NPI_WrFIFO_ALMOSTFULL = '0' then
						if(XIL_NPI_AddrAck = '1') then
							XIL_NPI_AddrReq <= '0';
							pim_state <= st_wr_wait_empty_high;
						end if;
					end if;
				when st_wr_wait_empty_high =>
					pim_status <= x"0a";
					if XIL_NPI_WrFIFO_EMPTY = '1' then
						add_cnt <= add_cnt + 1;
						if c_burst_repeat_cnt = (add_cnt + 1) then
							g_cnt <= g_cnt + 1;
							if (c_again_cnt = (g_cnt + 1)) and (c_again_cnt /= x"ffffffff") then
								fifo_rst <= '1';
								cnt_fifo_rst3clk <= (others => '0');
								pim_state <= st_fifo_rst3clk;
							else
								add_cnt <= (others => '0');	-- restart from start address
								pim_state <= st_wr_wait_full;		
							end if;
						else
							pim_state <= st_wr_wait_full;		
						end if;
					end if;
				-----------------------------------------------------------------------------
				-----------------------------------------------------------------------------
				-----------------------------------------------------------------------------
				when st_rd_wait_rdy =>
					pim_status <= x"0b";
					if XIL_NPI_RdFIFO_EMPTY = '1' and rd_fifo_prog_full_s = '0' then		
						XIL_NPI_RNW <= '1';
					    XIL_NPI_AddrReq <= '1';
						pim_state <= st_rd_add_req;
					end if;
				when st_rd_add_req =>
					pim_status <= x"0c";
					if(XIL_NPI_AddrAck = '1') then
					    XIL_NPI_AddrReq <= '0';
						burst_cnt<= x"f";
						pim_state <= st_rd_pop_data;
					end if;
				when st_rd_pop_data =>
					pim_status <= x"0d";
					if rd_fifo_full_s = '0' then  
						if rd_fifo_wr_en = '1' then 
							burst_cnt <= burst_cnt - 1;
							if burst_cnt = 0 then
								pim_state <= st_rd_next;
					 		end if;
						end if;
					end if;
				when st_rd_next =>
					pim_status <= x"0e";
					add_cnt <= add_cnt + 1;
					if c_burst_repeat_cnt = (add_cnt + 1) then
						g_cnt <= g_cnt + 1;
						if c_again_cnt = (g_cnt + 1) then
							pim_state <= st_rd_finish;
						else
							add_cnt <= (others => '0');	-- restart from start address
							pim_state <= st_rd_wait_rdy;
						end if;
					else
						pim_state <= st_rd_wait_rdy;
					end if;
				when st_rd_finish =>
					pim_status <= x"0f";
					if rd_fifo_empty_s = '1' then
						fifo_rst <= '1';
						cnt_fifo_rst3clk <= (others => '0');
						pim_state <= st_fifo_rst3clk;
					end if;
				when others => 
					pim_state <= st_idle;
				end case;
			end if;
		end if;
	end process; 

	XIL_NPI_WrFIFO_BE <= (others => '1'); 
	XIL_NPI_SIZE <= x"4";
	XIL_NPI_ADDR(31 downto 0) <= slv_wr_reg0(0 to 7) & "0" & add_cnt & "0000000"; 
	rd_fifo_din <= XIL_NPI_RdFIFO_Data(7 downto 0) & 
				   XIL_NPI_RdFIFO_Data(15 downto 8) & 
				   XIL_NPI_RdFIFO_Data(23 downto 16) & 
				   XIL_NPI_RdFIFO_Data(31 downto 24) & 
				   XIL_NPI_RdFIFO_Data(39 downto 32) & 
				   XIL_NPI_RdFIFO_Data(47 downto 40) & 
				   XIL_NPI_RdFIFO_Data(55 downto 48) & 
				   XIL_NPI_RdFIFO_Data(63 downto 56);

	XIL_NPI_WrFIFO_Data <= wr_fifo_dout(7 downto 0) & 
						   wr_fifo_dout(15 downto 8) & 
						   wr_fifo_dout(23 downto 16) & 
						   wr_fifo_dout(31 downto 24) & 
						   wr_fifo_dout(39 downto 32) & 
						   wr_fifo_dout(47 downto 40) & 
						   wr_fifo_dout(55 downto 48) & 
						   wr_fifo_dout(63 downto 56); 

	process(XIL_NPI_RdFIFO_Latency, XIL_NPI_RdFIFO_Pop_s, XIL_NPI_RdFIFO_Pop_d0, XIL_NPI_RdFIFO_Pop_d1)   
	begin
		case XIL_NPI_RdFIFO_Latency is
			when "00" => rd_fifo_wr_en <= XIL_NPI_RdFIFO_Pop_s;
			when "01" => rd_fifo_wr_en <= XIL_NPI_RdFIFO_Pop_d0;
			when "10" => rd_fifo_wr_en <= XIL_NPI_RdFIFO_Pop_d1;
			when others => rd_fifo_wr_en <= XIL_NPI_RdFIFO_Pop_s;
		end case;
	end process;

	process(XIL_NPI_Clk)
	begin
		if XIL_NPI_Clk'event and XIL_NPI_Clk = '1' then
			XIL_NPI_RdFIFO_Pop_d0 <= XIL_NPI_RdFIFO_Pop_s; 
			XIL_NPI_RdFIFO_Pop_d1 <= XIL_NPI_RdFIFO_Pop_d0;
		end if;
	end process;
	

	XIL_NPI_RdFIFO_Pop_s <= (not XIL_NPI_RdFIFO_Empty) and (not rd_fifo_prog_full_s) when pim_state = st_rd_pop_data else '0';
	XIL_NPI_RdFIFO_Pop <= XIL_NPI_RdFIFO_Pop_s;         

	slv_rd_reg0 <= XIL_NPI_InitDone & 
				   wr_fifo_full_s & 
				   rd_fifo_full_s & 
				   XIL_NPI_WrFIFO_EMPTY & 
				   XIL_NPI_RdFIFO_EMPTY & 
				   npi_wr_ready_s & 
				   npi_rd_ready_s & '0' & x"000000"; 			
	slv_rd_reg1 <= x"0000" & c_burst_repeat_cnt; 
	slv_rd_reg2 <= x"0000" & add_cnt; 
	slv_rd_reg3 <= x"000000" & pim_status;


	npi_rd_ready <= npi_rd_ready_s;
	npi_wr_ready <= npi_wr_ready_s;
	----------------------------------------------------------------------------------------------------------------------------------------------
	-- command flag
	----------------------------------------------------------------------------------------------------------------------------------------------
	start_wr_cmd <= '1' when ((slv_reg_write_sel = "000100") and (Bus2IP_Data(24 to 31) = start_wr_cmd_patt)) else '0';
	start_rd_cmd <= '1' when ((slv_reg_write_sel = "000100") and (Bus2IP_Data(24 to 31) = start_rd_cmd_patt)) else '0';
	user_rst_cmd <= '1' when ((slv_reg_write_sel = "000100") and (Bus2IP_Data(24 to 31) = user_rst_cmd_patt)) else '0';
end IMP;
